@@ -38,7 +38,7 @@ def pytest_configure(config):
     config.addinivalue_line('markers', "flakes: Tests which run flake.")
 
 
-class FlakesPlugin(object):
+class FlakesPlugin:
     def __init__(self, config):
         self.ignore = Ignorer(config.getini("flakes-ignore"))
         self.mtimes = config.cache.get(HISTKEY, {})
@@ -48,13 +48,9 @@ class FlakesPlugin(object):
         if config.option.flakes and isPythonFile(path.strpath):
             flakesignore = self.ignore(path)
             if flakesignore is not None:
-                if hasattr(FlakesItem, 'from_parent'):
-                    item = FlakesItem.from_parent(parent,
-                                                  fspath=path,
-                                                  flakesignore=flakesignore)
-                else:
-                    item = FlakesItem(path, parent, flakesignore)
-                return item
+                return FlakesFile.from_parent(parent,
+                                              fspath=path,
+                                              flakesignore=flakesignore)
 
     def pytest_sessionfinish(self, session):
         session.config.cache.set(HISTKEY, self.mtimes)
@@ -64,15 +60,24 @@ class FlakesError(Exception):
     """ indicates an error during pyflakes checks. """
 
 
-class FlakesItem(pytest.Item, pytest.File):
+class FlakesFile(pytest.File):
+    def __init__(self, *k,  flakesignore, **kw):
+        super().__init__(*k, **kw)
+        self.flakesignore = flakesignore
 
-    def __init__(self, fspath, parent, flakesignore):
-        super(FlakesItem, self).__init__(fspath, parent)
+    def collect(self):
+        return [FlakesItem.from_parent(self, name="flake-8")]
+
+
+class FlakesItem(pytest.Item):
+
+    def __init__(self, *k, **kw):
+        super().__init__(*k, **kw)
         if hasattr(self, 'add_marker'):
             self.add_marker("flakes")
         else:
             self.keywords["flakes"] = True
-        self.flakesignore = flakesignore
+        self.flakesignore = self.parent.flakesignore
 
     def setup(self):
         flakesmtimes = self.config._flakes.mtimes
@@ -92,7 +97,7 @@ class FlakesItem(pytest.Item, pytest.File):
     def repr_failure(self, excinfo):
         if excinfo.errisinstance(FlakesError):
             return excinfo.value.args[0]
-        return super(FlakesItem, self).repr_failure(excinfo)
+        return super().repr_failure(excinfo)
 
     def reportinfo(self):
         if self.flakesignore:
@@ -101,6 +106,12 @@ class FlakesItem(pytest.Item, pytest.File):
             ignores = ""
         return (self.fspath, -1, "pyflakes-check%s" % ignores)
 
+    def collect(self):
+        """ returns a list of children (items and collectors)
+            for this collection node.
+        """
+        return (self,)
+
 
 class Ignorer:
     def __init__(self, ignorelines, coderex=re.compile(r"[EW]\d\d\d")):
@@ -152,10 +163,10 @@ def check_file(path, flakesignore):
                 offset = offset - (len(text) - len(line))
 
             msg = '%s:%d: %s' % (filename, lineno, value.args[0])
-            msg = "%s\n%s" % (msg, line)
+            msg = "{}\n{}".format(msg, line)
 
             if offset is not None:
-                msg = "%s\n%s" % (msg, "%s^" % (" " * offset))
+                msg = "{}\n{}".format(msg, "%s^" % (" " * offset))
             errors.append(msg)
         return 1, errors
     else:
@@ -167,7 +178,7 @@ def check_file(path, flakesignore):
             if warning.__class__.__name__ in flakesignore or is_ignored_line(lines[warning.lineno - 1].strip()):
                 continue
             errors.append(
-                '%s:%s: %s\n%s' % (
+                '{}:{}: {}\n{}'.format(
                     warning.filename,
                     warning.lineno,
                     warning.__class__.__name__,
